from typing import Annotated

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.destination_table_info import DestinationTableInfoModel
from adsdatahub.restapi.schemas.output_artifacts import OutputArtifactsModel
from adsdatahub.restapi.schemas.privacy_message import PrivacyMessageModel


class QueryResponseModel(ExtraAllowModel):
    """
    クエリ実行ジョブが成功すると、レスポンスが返されます。
    これは、クエリ実行リクエストによって返される google.longrunning.Operation のレスポンス フィールドに保存されます。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/QueryResponse?hl=ja
    """

    type: Annotated[
        str,
        Field(alias="@type"),
    ]
    """
    タイプを識別する URI を含むフィールド。
    """

    row_count: Annotated[
        int,
        Field(alias="rowCount"),
    ]
    """
    最終出力テーブルの行数。

    ラップされた値は、行数が 0 のケースと、この情報が利用できないケースを区別するために使用されます。
    """

    destination_tables: Annotated[
        list[DestinationTableInfoModel],
        Field(alias="destinationTables", default_factory=list),
    ]
    """
    実行結果のテーブル情報。

    ドキュメントには記載されていないが、実際のレスポンスには含まれる。
    ノイズなどの情報が含まれている。
    """

    output_artifacts: Annotated[
        OutputArtifactsModel,
        Field(alias="outputArtifacts"),
    ]
    """
    クエリ実行中に生成された出力アーティファクト。
    """

    privacy_messages: Annotated[
        list[PrivacyMessageModel],
        Field(alias="privacyMessages", default_factory=list),
    ]
    """
    プライバシー関連の情報または警告メッセージのリスト。
    """



from enum import Enum


class ShareType(str, Enum):
    """
    サポートされているクエリ共有のタイプ。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/QueryShare?hl=ja
    """

    SHARE_TYPE_UNSPECIFIED = "SHARE_TYPE_UNSPECIFIED"
    """クエリの共有タイプが指定されていません。"""

    MANAGED_CUSTOMERS = "MANAGED_CUSTOMERS"
    """クエリは、Ads Data Hub ユーザー、またはお客様が管理している Ads Data Hub ユーザーのどちらからでも実行できます。"""



from typing_extensions import Literal, TypedDict


class TypeDict(TypedDict):
    # See: https://developers.google.com/ads-data-hub/marketers/guides/run-queries?hl=ja#parameter_types
    type: Literal[
        "INT64",
        "FLOAT64",
        "BOOL",
        "STRING",
        "DATE",
        "TIMESTAMP",
    ]


class ArrayTypeDict(TypedDict):
    arrayType: "FieldTypeDict"


class StructTypeDict(TypedDict):
    structType: list["StructFieldDict"]


class StructFieldDict(TypedDict):
    fieldName: str
    fieldType: "FieldTypeDict"


FieldTypeDict = TypeDict | ArrayTypeDict | StructTypeDict
"""
BigQuery のフィールド タイプを定義します。

Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/FieldType?hl=ja
"""


class TypeModel(TypedDict):
    # See: https://developers.google.com/ads-data-hub/marketers/guides/run-queries?hl=ja#parameter_types
    type: Literal[
        "INT64",
        "FLOAT64",
        "BOOL",
        "STRING",
        "DATE",
        "TIMESTAMP",
    ]


class ArrayTypeModel(TypedDict):
    arrayType: "FieldTypeModel"


class StructTypeModel(TypedDict):
    structType: list["StructFieldModel"]


class StructFieldModel(TypedDict):
    fieldName: str
    fieldType: "FieldTypeModel"


FieldTypeModel = TypeModel | ArrayTypeModel | StructTypeModel

FieldType = FieldTypeDict | FieldTypeModel



from typing import Annotated, NotRequired, TypedDict

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.analysis_query import AnalysisQueryOptionalTitleRequest
from adsdatahub.restapi.schemas.filtered_row_summary import FilteredRowSummaryModel
from adsdatahub.restapi.schemas.query_execution_spec import QueryExecutionSpecRequest
from adsdatahub.restapi.schemas.query_performance_info import QueryPerformanceInfoModel


class AnalysisQueriesValidateQueryParams(TypedDict):
    """
    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/validate?hl=ja#request-body
    """

    query: AnalysisQueryOptionalTitleRequest
    """
    検証するクエリ。
    """

    adsDataCustomerId: NotRequired[str | None]
    """
    （省略可）広告データの検証と一時テーブル アクセスに使用する、リンクされた Ads Ads Hub お客様 ID。

    デフォルトでは、リンクされているすべての顧客に表示されます。
    非推奨です。query_execution_spec 内で adsDataCustomerId を使用します。
    """

    matchDataCustomerId: NotRequired[str | None]
    """
    （省略可）マッチデータと検証テーブルへのアクセスの検証に使用する、リンクされた Ads Ads Hub お客様 ID。

    デフォルトでは、リンクされているすべての顧客に表示されます。
    非推奨です。query_execution_spec 内の match_table_customer_id を使用します。
    """

    spec: NotRequired[QueryExecutionSpecRequest | None]
    """
    （省略可）query_execution_spec 内の重複するフィールドが、外側のフィールドより優先されます。
    """

    includePerformanceInfo: NotRequired[bool | None]
    """
    （省略可）true の場合、BigQuery を呼び出してクエリをドライランして、パフォーマンス情報を収集します。

    クエリを検証するだけの場合よりも時間がかかる可能性があります。
    ドライランには、startDate、endDate、query_execution_spec 内のパラメータを設定する必要があります。
    """


class AnalysisQueriesValidateResponseBodyModel(ExtraAllowModel):
    """
    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/validate?hl=ja#response-body
    """

    processed_bytes: Annotated[int | None, Field(alias="processedBytes")] = None
    """
    サポートが終了し、代わりに queryPerformanceInfo を使用してください。

    このクエリが読み取るデータのバイト数。
    この計算方法の詳細については、https://cloud.google.com/bigquery/pricing#data をご覧ください。
    includePerformanceInfo が true の場合に返されます。
    Google は processBytes バイトを MB 単位に丸め、次にバイト単位に変換します。
    -1 は、このクエリで取得できなかったことを意味します。
    """

    query_performance_info: Annotated[
        QueryPerformanceInfoModel | None, Field(alias="queryPerformanceInfo")
    ] = None
    """
    includePerformanceInfo が true の場合に返されます。
    """

    filtered_row_summary: Annotated[
        FilteredRowSummaryModel,
        Field(alias="filteredRowSummary"),
    ]
    """
    クエリ実行時に使用される、フィルタされた行のサマリー。

    詳しくは、https://developers.google.com/ads-data-hub/guides/filtered-row-summary をご覧ください。
    """



from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.column import ColumnModel
from adsdatahub.restapi.schemas.query_type import QueryType


class TempTableModel(ExtraAllowModel):
    """
    Ads Data Hub クエリによって作成された一時テーブルを定義します。
    一時テーブルは、Ads Data Hub クエリで CREATE TABLE temp_table AS (...) を使用して作成されます。
    一時テーブルは、更新が必要になるまで 72 時間アクティブ状態が維持されます。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.tempTables?hl=ja#TempTable
    """

    name: str
    """
    テーブルを一意に識別する名前。

    形式は customer/[お客様 ID]/tempTables/[analysis_query_id] です。
    リソース ID はサーバーによって生成されます。
    """

    tablePath: str
    """
    完全修飾されたテーブルパス。例: 'tmp.table'
    """

    adsDataCustomerId: str
    """
    リンクされた一時 Data Hub のお客様 ID（元の一時テーブルのクエリの実行時に使用）。
    """

    matchDataCustomerId: str
    """
    リンクされた一時 Data Hub のお客様 ID（元の一時テーブルのクエリの実行時に使用）。
    """

    queryType: QueryType
    """
    テーブルを作成したクエリのタイプ。

    一時テーブルにアクセスできるのは、usageQueryTypes が設定されていない限り、同じタイプのクエリに限られます。
    """

    usableQueryTypes: list[QueryType]
    """
    この一時テーブルにアクセスできるクエリの種類。

    空の場合、この一時テーブルにはその queryType と同じ型のクエリのみがアクセスできます。
    """

    columns: list[ColumnModel]
    """
    テーブル列のリスト。
    """

    createTime: str
    """
    一時テーブルが作成されたときのタイムスタンプ（マイクロ単位）。

    RFC3339 UTC の Zulu 形式のタイムスタンプ。
    ナノ秒単位で、小数点以下は 9 桁までとなります。
    （例: "2014-10-02T15:01:23Z"、"2014-10-02T15:01:23.045123456Z"）。
    """

    operation: str
    """
    一時テーブルを作成したオペレーション。

    形式: operations/[jobId]
    """



import os
from typing import TYPE_CHECKING, Any, TypedDict, Union

import google.api_core.client_info
import google.api_core.client_options
import google.api_core.exceptions
import google.auth.credentials
import google.auth.transport.requests
from google.auth import environment_vars
from google.auth.credentials import AnonymousCredentials, Credentials
from google.cloud.client import ClientWithProject
from typing_extensions import Unpack, override

from .client import Client

if TYPE_CHECKING:
    import adsdatahub.restapi.http

_marker: object = object()


ADSDATAHUB_EMULATOR_ENV_VAR = "ADSDATAHUB_EMULATOR_HOST"
"""Environment variable defining host for Ads Data Hub emulator."""

_API_ENDPOINT_OVERRIDE_ENV_VAR = "API_ENDPOINT_OVERRIDE"
"""This is an experimental configuration variable. Use api_endpoint instead."""

_DEFAULT_ADSDATAHUB_HOST = os.getenv(
    _API_ENDPOINT_OVERRIDE_ENV_VAR, "https://adsdatahub.googleapis.com"
)


class RealRestApiClientConstructerKwargs(TypedDict, total=False):
    project: str | object | None
    credentials: Credentials | None
    client_info: google.api_core.client_info.ClientInfo | None
    client_options: Union[
        google.api_core.client_options.ClientOptions,
        dict[str, Any],
        None,
    ]
    _http: Any


class RealClient(Client, ClientWithProject):
    SCOPE = ("https://www.googleapis.com/auth/adsdatahub",)

    def __init__(self, **kwargs: Unpack[RealRestApiClientConstructerKwargs]) -> None:
        project = kwargs.pop("project", _marker)
        if project is None:
            no_project = True
            project = "<none>"
        else:
            no_project = False

        if project is _marker:
            project = None

        client_info = kwargs.pop("client_info", None)
        client_options = kwargs.pop("client_options", None)
        credentials = kwargs.pop("credentials", None)
        _http = kwargs.pop("_http", None)

        self._initial_client_info = client_info
        self._initial_client_options = client_options
        self._initial_credentials = credentials

        kw_args: dict[str, Any] = {"client_info": client_info}

        # `api_endpoint` should be only set by the user via `client_options`,
        # or if the _get_storage_host() returns a non-default value (_is_emulator_set).
        # `api_endpoint` plays an important role for mTLS, if it is not set,
        # then mTLS logic will be applied to decide which endpoint will be used.
        storage_host = _get_adsdatahub_host()
        _is_emulator_set = storage_host != _DEFAULT_ADSDATAHUB_HOST
        kw_args["api_endpoint"] = storage_host if _is_emulator_set else None

        if client_options:
            if isinstance(client_options, dict):
                client_options = google.api_core.client_options.from_dict(
                    client_options
                )
            if client_options.api_endpoint:
                api_endpoint = client_options.api_endpoint
                kw_args["api_endpoint"] = api_endpoint

        # If a custom endpoint is set, the client checks for credentials
        # or finds the default credentials based on the current environment.
        # Authentication may be bypassed under certain conditions:
        # (1) STORAGE_EMULATOR_HOST is set (for backwards compatibility), OR
        # (2) use_auth_w_custom_endpoint is set to False.
        if kw_args["api_endpoint"] is not None:
            if _is_emulator_set:
                if credentials is None:
                    credentials = AnonymousCredentials()
                if project is None:
                    project = _get_environ_project()
                if project is None:
                    no_project = True
                    project = "<none>"

        ClientWithProject.__init__(
            self,
            project=project,
            credentials=credentials,
            client_options=client_options,
            _http=_http,
        )

        if _http is None and self._credentials is not None:
            if not self._credentials.valid:
                self._credentials.refresh(google.auth.transport.requests.Request())

            import adsdatahub.restapi.http

            self._http_internal = adsdatahub.restapi.http.RealClient(
                headers={
                    "Authorization": f"Bearer {self._credentials.token}",
                },
                timeout=60,
            )

        if no_project:
            self.project = None

    @property
    @override
    def _http(self) -> "adsdatahub.restapi.http.Client":
        return self._http_internal


def _get_environ_project():
    return os.getenv(
        environment_vars.PROJECT,
        os.getenv(environment_vars.LEGACY_PROJECT),
    )


def _get_adsdatahub_host():
    return os.environ.get(ADSDATAHUB_EMULATOR_ENV_VAR, _DEFAULT_ADSDATAHUB_HOST)



from typing import TYPE_CHECKING, Literal, TypedDict, Unpack

from adsdatahub.restapi._helpers import (
    convert_json_value,
)
from adsdatahub.restapi.resources.analysis_query.start import (
    AnalysisQueryStartRequestBody,
)
from adsdatahub.restapi.schemas.analysis_query import (
    AnalysisQueryModel,
    AnalysisQueryRequestDict,
    AnalysisQueryRequestModel,
)
from adsdatahub.restapi.schemas.analysis_query_metadata import (
    AnalysisQueryMetadataModel,
)
from adsdatahub.restapi.schemas.operation import OperationModel
from adsdatahub.restapi.schemas.query_execution_spec import (
    QueryExecutionSpecRequestModel,
)
from adsdatahub.types import AnalysisQueryId, CustomerId

if TYPE_CHECKING:
    import adsdatahub.restapi.http

ResourceName = Literal[
    "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries/{analysis_query_id}"
]
RESOURCE_NAME: ResourceName = "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries/{analysis_query_id}"


class PathParameters(TypedDict):
    customer_id: CustomerId
    analysis_query_id: AnalysisQueryId


class Resource:
    def __init__(
        self, http: "adsdatahub.restapi.http.Client", path_parameters: PathParameters
    ) -> None:
        self._http = http
        self._base_url = RESOURCE_NAME.format(**path_parameters)

    def delete(self) -> None:
        """
        分析クエリを削除します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/delete?hl=ja
        """
        return self._http.request("DELETE", self._base_url)

    def get(self) -> AnalysisQueryModel:
        """
        リクエストされた分析クエリを取得します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/get?hl=ja
        """
        return self._http.request(
            "GET",
            self._base_url,
            AnalysisQueryModel,
        )

    def patch(
        self, request_body: AnalysisQueryRequestModel | AnalysisQueryRequestDict
    ) -> AnalysisQueryModel:
        """
        既存の分析クエリを更新します。部分更新はサポートされています。次のクエリ フィールドは、この方法では更新できないため、無視されます。

        - queryState
        - createTime
        - createEmail（作成メール）
        - updateTime
        - updateEmail

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/patch?hl=ja
        """
        return self._http.request(
            "PATCH",
            self._base_url,
            AnalysisQueryModel,
            json=(
                AnalysisQueryRequestModel.model_validate(request_body)
                if isinstance(request_body, dict)
                else request_body
            ).model_dump(),
        )

    def start(
        self, request_body: AnalysisQueryStartRequestBody
    ) -> OperationModel[AnalysisQueryMetadataModel]:
        """
        保存された分析クエリの実行を開始します。
        結果は、指定した BigQuery 宛先テーブルに書き込まれます。
        返されたオペレーション名を使用して、クエリ完了ステータスをポーリングできます。

        Refarence: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/start?hl=ja
        """

        return self._http.request(
            "POST",
            f"{self._base_url}:start",
            OperationModel[AnalysisQueryMetadataModel],
            json=convert_json_value(
                request_body,
                model_map={
                    "spec": QueryExecutionSpecRequestModel,
                },
            ),
        )


class MockResource:
    def __init__(
        self,
        mock_client: "adsdatahub.restapi.MockClient",
        **path_parameters: Unpack[PathParameters],
    ) -> None:
        self._mock_client = mock_client
        self._base_url = RESOURCE_NAME.format(**path_parameters)

    def delete(
        self, response: None | Exception = None
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response("DELETE", self._base_url, response)

        return self._mock_client

    def get(
        self, response: AnalysisQueryModel | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response("GET", self._base_url, response)

        return self._mock_client

    def patch(
        self, response: AnalysisQueryModel | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response("PATCH", self._base_url, response)

        return self._mock_client

    def start(
        self, response: OperationModel[AnalysisQueryMetadataModel] | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response(
            "POST", f"{self._base_url}:start", response
        )

        return self._mock_client



from collections import UserList
from typing import Annotated

from pydantic import Field, PlainSerializer, PlainValidator

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.column_info import ColumnInfoModel
from adsdatahub.restapi.schemas.table_noise_impact import TableNoiseImpact


class ColumnInfoModelList(UserList[ColumnInfoModel]):
    def __getitem__(self, key: int | str) -> ColumnInfoModel:  # type: ignore[override]
        if isinstance(key, int):
            return super().__getitem__(key)

        else:
            if column := self.get(key):
                return column
            raise KeyError(f"Column not found: {key}")

    def get(
        self, key: int | str, default: ColumnInfoModel | None = None
    ) -> ColumnInfoModel | None:
        if isinstance(key, int):
            if len(self) > key:
                return self[key]

            else:
                return default

        else:
            for column in self:
                if column.name == key:
                    return column

            else:
                return default


def _validate_column_info_list(value: list[ColumnInfoModel]) -> ColumnInfoModelList:
    return ColumnInfoModelList(
        [ColumnInfoModel.model_validate(model) for model in value]
    )


def _serialize_column_info_list(value: ColumnInfoModelList):
    return [v for v in value]


class DestinationTableInfoModel(ExtraAllowModel):
    """
    Metadata of an exported query output table.

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/DestinationTableInfo
    """

    table_path: Annotated[str, Field(alias="tablePath")]
    """
    Table path in the customer BigQuery project.
    """

    row_count: Annotated[int, Field(alias="rowCount")] = 0
    """
    Number of rows in the result.
    """

    columns: Annotated[
        ColumnInfoModelList,
        Field(default_factory=list),
        PlainValidator(_validate_column_info_list),
        PlainSerializer(_serialize_column_info_list),
    ]
    """
    Information about columns in result.
    """

    noise_impact: Annotated[TableNoiseImpact, Field(alias="noiseImpact")]
    """
    The noise impact message for this table.
    """

    impact_percentage: Annotated[float | None, Field(alias="impactPercentage")] = None
    """
    If applicable, the percent of cells in this table which are not greatly affected by noise.
    """



from typing import Unpack, cast

from google.cloud import bigquery
from typing_extensions import override

import adsdatahub.restapi
from adsdatahub.client.client import Client
from adsdatahub.restapi.real_client import RealRestApiClientConstructerKwargs
from adsdatahub.types import CustomerId


class RealClient(Client):
    def __init__(
        self,
        restapi_client: adsdatahub.restapi.RealClient | None = None,
        bigquery_client: bigquery.Client | None = None,
        **kwargs: Unpack[RealRestApiClientConstructerKwargs],
    ) -> None:
        if not restapi_client:
            restapi_client = cast(
                adsdatahub.restapi.RealClient,
                adsdatahub.restapi.RealClient(**kwargs),
            )

        if not bigquery_client:
            bigquery_client = bigquery.Client(**kwargs)

        self.restapi = restapi_client
        self.bigquery_client = bigquery_client

    @override
    def customer(self, customer_id: CustomerId):
        from adsdatahub.client.real_customer import RealCustomerClient

        return RealCustomerClient(self, customer_id)



from enum import Enum


class QueryType(str, Enum):
    """
    クエリのタイプ。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.tempTables?hl=ja#QueryType
    """

    QUERY_TYPE_UNSPECIFIED = "QUERY_TYPE_UNSPECIFIED"
    """クエリの種類が指定されていません。"""

    ANALYSIS = "ANALYSIS"
    """AnalysisQuery。"""

    USER_LIST = "USER_LIST"
    """UserListQuery。"""

    SPECIAL_ENDPOINT = "SPECIAL_ENDPOINT"
    """特殊なエンドポイントによってトリガーされたクエリ。"""



import datetime
from typing import Annotated

from pydantic import Field, field_serializer
from typing_extensions import NotRequired, TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel, ExtraForbidModel
from adsdatahub.restapi.schemas.date import DateDict, DateModel
from adsdatahub.restapi.schemas.parameter_value import (
    ParameterValueDict,
    ParameterValueModel,
)


class QueryExecutionSpecRequestDict(TypedDict):
    """クエリ実行パラメータを定義します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/QueryExecutionSpec?hl=ja
    """

    adsDataCustomerId: NotRequired[str | None]
    """
    クエリで使用されている広告データを所有している、
    リンクされた Ads Data Hub お客様 ID。
    指定されていない場合、クエリは、Ads Data Hub ユーザーが所有する広告データを使用して実行されます。
    """

    matchDataCustomerId: NotRequired[str | None]
    """
    クエリの一致データを所有している、リンクされた Ads Data Hub お客様 ID。
    指定されていない場合、クエリは、該当する場合は Ads Data Hub お客様 ID が所有するマッチデータを使用して実行されます。
    """

    # TODO: converter
    startDate: DateDict | datetime.date | str
    """クエリの開始日（この日付を含む）。"""

    # TODO: converter
    endDate: DateDict | datetime.date | str
    """クエリの終了日（その日を含む）。"""

    timeZone: NotRequired[str | None]
    """クエリの開始日と終了日のタイムゾーン。指定しない場合、デフォルトは 'UTC' です。"""

    parameterValues: NotRequired[dict[str, ParameterValueDict] | None]
    """クエリで想定されるその他のパラメータ。各パラメータ名をそのバインドされた値にマッピングします。"""

    jobId: NotRequired[str | None]
    """
    クエリ オペレーションのジョブ ID。
    結果として得られるオペレーションは、"operations/[jobId]" という名前になります（例: "operations/job_123"）。
    同じジョブ ID のオペレーションがすでに存在する場合は、エラーが発生します。
    指定されていない場合、サーバーによってジョブ ID が生成されます。
    """


class QueryExecutionSpecRequestModel(ExtraForbidModel):
    ads_data_customer_id: Annotated[str | None, Field(alias="adsDataCustomerId")] = None
    match_data_customer_id: Annotated[
        str | None, Field(alias="matchDataCustomerId")
    ] = None
    start_date: Annotated[DateModel | datetime.date | str, Field(alias="startDate")]
    end_date: Annotated[DateModel | datetime.date | str, Field(alias="endDate")]
    time_zone: Annotated[str | None, Field(alias="timeZone")] = None
    parameter_values: Annotated[
        dict[str, ParameterValueModel],
        Field(alias="parameterValues", default_factory=dict),
    ]
    job_id: Annotated[str | None, Field(alias="jobId")] = None

    @field_serializer("start_date", "end_date")
    def serialize_dt(self, date: DateModel | datetime.date | str) -> DateDict:
        if isinstance(date, str):
            date = datetime.date.fromisoformat(date)

        if isinstance(date, datetime.date):
            return DateDict(year=date.year, month=date.month, day=date.day)

        else:
            return DateDict(year=date.year, month=date.month, day=date.day)


class QueryExecutionSpecModel(ExtraAllowModel):
    adsDataCustomerId: str
    matchDataCustomerId: str
    startDate: DateModel
    endDate: DateModel
    timeZone: str
    parameterValues: dict[str, ParameterValueModel]
    jobId: str


QueryExecutionSpecRequest = (
    QueryExecutionSpecRequestDict | QueryExecutionSpecRequestModel
)



from typing import Annotated

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel


class QueryPerformanceInfoModel(ExtraAllowModel):
    """
    クエリ実行のパフォーマンス情報。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/QueryPerformanceInfo?hl=ja
    """

    zero_mb: Annotated[bool, Field(alias="zeroMb")] = False
    """
    処理されたデータが 0 バイトの場合は true。
    """

    less_than_one_mb: Annotated[bool, Field(alias="lessThanOneMb")] = False
    """
    処理されたデータが 1 MiB 未満の場合は true。
    """

    processed_mb: Annotated[int | None, Field(alias="processedMb")] = None
    """
    このクエリで読み取られる MiB データ。
    データサイズが 1 MiB 以上の場合に設定します。-1 は、このクエリで処理されたバイト数をフェッチできなかったことを意味します。
    """



from enum import Enum


class NoiseImpact(str, Enum):
    """
    LINT.IfChange(ColumnNoiseImpact) Impact of noise on column result confidence for UI warnings.

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/ColumnNoiseImpact
    """

    COLUMN_NOISE_IMPACT_UNSPECIFIED = "COLUMN_NOISE_IMPACT_UNSPECIFIED"
    """
    No noise impact specified.
    """

    NOT_NOISED = "NOT_NOISED"
    """
    Column is not noised.
    """

    LOW_IMPACT = "LOW_IMPACT"
    """
    Noise has a low impact on the column's result confidence.
    """

    HIGH_IMPACT = "HIGH_IMPACT"
    """
    Noise has a high impact on the column's result confidence.
    """



from enum import Enum


class TableNoiseImpact(str, Enum):
    """
    Impact of noise on table result confidence for UI warnings

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/TableNoiseImpact
    """

    TABLE_NOISE_IMPACT_UNSPECIFIED = "TABLE_NOISE_IMPACT_UNSPECIFIED"
    """
    No noise impact specified
    """

    NOT_NOISED = "NOT_NOISED"
    """
    Table is not noised.
    """

    LOW_IMPACT = "LOW_IMPACT"
    """
    Noise has a low impact on the table's result confidence.
    """

    HIGH_IMPACT = "HIGH_IMPACT"
    """
    Noise has a high impact on the table's result confidence.
    """

    LOW_MEDIUM_IMPACT = "LOW_MEDIUM_IMPACT"
    """
    Noise has a low to medium impact on the table's result confidence.
    """

    MEDIUM_IMPACT = "MEDIUM_IMPACT"
    """
    Noise has a medium impact on the table's result confidence.
    """

    VERY_HIGH_IMPACT = "VERY_HIGH_IMPACT"
    """
    Noise has a very high impact on the table's result confidence.
    """



import datetime
from typing import NotRequired, TypedDict


class OperationWaitRequestBody(TypedDict):
    """
    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/wait?hl=ja#request-body
    """

    timeout: NotRequired[datetime.timedelta | str | int | float | None]
    """
    タイムアウトするまでの最大待機時間。

    空白のままにした場合、待機時間は基になる HTTP/RPC プロトコルによって許可される最長の時間になります。
    RPC コンテキストの期限も指定されている場合は、短い方が使用されます。

    小数点以下 9 桁まで、「s」で終わる秒単位の期間（例: "3.5s"）。
    """



from typing import TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.merge_type import MergeType
from adsdatahub.restapi.schemas.parameter_value import (
    ParameterValueDict,
    ParameterValueModel,
)


class MergeColumnDict(TypedDict):
    """
    単一の出力列の手順をマージします。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#MergeColumn
    """

    type: MergeType
    """
    列の値を組み合わせるために使用するメソッドです。
    """

    value: ParameterValueDict
    """
    使用する定数値（CONSTANT マージタイプでのみ有効）。
    """


class MergeColumnModel(ExtraAllowModel):
    """
    単一の出力列の手順をマージします。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#MergeColumn
    """

    type: MergeType
    """
    列の値を組み合わせるために使用するメソッドです。
    """

    value: ParameterValueModel
    """
    使用する定数値（CONSTANT マージタイプでのみ有効）。
    """



from typing import Annotated, NotRequired, TypedDict

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.analysis_query import AnalysisQueryModel


class AnalysisQueryListQueryParams(TypedDict):
    pageSize: NotRequired[int | None]
    """返される最大アイテム数。0 の場合、サーバーは返されるクエリの数を決定します。"""

    pageToken: NotRequired[str | None]
    """前の呼び出しによって返されたページトークン。次のページの結果を返すために使用されます。"""

    filter: NotRequired[str | None]
    """
    レスポンスをフィルタします。

    次のフィールド / 形式を使用します。

    ```
    name=”customers/271828/analysisQueries/pi314159265359”
    title=”up_and_right”
    queryText="SELECT LN(2.7182818284);"
    queryState="RUNNABLE"
    updateTime>unix_seconds
    updateEmail=”abc@gmail.com”
    createTime>unix_seconds
    createEmail=”abc@gmail.com”
    ```
    """


class AnalysisQueryListResponse(ExtraAllowModel):
    queries: list[AnalysisQueryModel]
    """
    クエリのリスト。
    """

    next_page_token: Annotated[str | None, Field(alias="nextPageToken")] = None
    """
    次の結果ページのリクエストに使用できるトークン。

    他に結果がない場合、このフィールドは空です。
    """



import typing
from types import NoneType

from typing_extensions import override

from adsdatahub.exceptions import (
    AdsDataHubMockDataTypeError,
    AdsDataHubMockStoreDataEmptyError,
    AdsDataHubMockStoreKeyError,
)
from adsdatahub.restapi._helpers import (
    GenericResponseBody,
)
from adsdatahub.restapi.http.client import HttpRequestKwargs
from adsdatahub.restapi.schemas._model import Model
from adsdatahub.types import TimeoutTypes, URLTypes

from .client import Client


class StoreKey(typing.NamedTuple):
    method: str
    url: URLTypes


_StoreData = list[tuple[StoreKey, Model | None | Exception]]


class MockClient(Client):
    _store: _StoreData

    __slots__ = ("_store",)

    @property
    def store(self) -> _StoreData:
        if not hasattr(self, "_store"):
            self._store = []

        return self._store

    @property
    @override
    def timeout(self) -> TimeoutTypes:
        return 0

    @timeout.setter
    @override
    def timeout(self, value: TimeoutTypes) -> None:
        pass

    @typing.overload
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: type[GenericResponseBody],
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> GenericResponseBody: ...

    @typing.overload
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: None = None,
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> None: ...

    @override
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: typing.Optional[type[GenericResponseBody]] = None,
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> typing.Optional[GenericResponseBody]:
        if len(self.store) == 0:
            raise AdsDataHubMockStoreDataEmptyError()
        expected_key = StoreKey(method, url)
        key, response = self.store.pop(0)

        if key != expected_key:
            raise AdsDataHubMockStoreKeyError(key, expected_key)

        if isinstance(response, Exception):
            raise response

        if response_body_type is None:
            if response is not None:
                raise AdsDataHubMockDataTypeError(response.__class__, NoneType)

        elif not isinstance(response, response_body_type):
            raise AdsDataHubMockDataTypeError(response.__class__, response_body_type)

        else:
            return response

    def inject_response(
        self,
        method: str,
        url: URLTypes,
        response: Model | None | Exception,
    ) -> "MockClient":
        self.store.append((StoreKey(method, url), response))

        return self



import datetime
from typing import assert_never

from adsdatahub.restapi.schemas.field_type import (
    ArrayTypeDict,
    FieldTypeDict,
    TypeDict,
)
from adsdatahub.restapi.schemas.parameter_type import (
    ParameterTypeDict,
)
from adsdatahub.restapi.schemas.parameter_value import (
    ArrayValueDict,
    ArrayValueValuesDict,
    ParameterValueDict,
    ValueDict,
)

# TODO: 配列の型、デフォルト値などをサポートする必要があるが、後回し。
#
# TODO: デフォルト値や NULL のサポートをするためには、 dataclass や pydantic を利用する必要がある。
#
# NOTE: ドキュメントの記載は一貫性がない。
#       1. STRING または INT64 のプリミティブ型に加えて、配列と構造体をサポートしているように見える。
#          https://developers.google.com/ads-data-hub/reference/rest/v1/FieldType?hl=ja
#
#       2. AdsDataHub のコンソールからはさらに多くのパラメータの型を扱うことができるようだ。
#          https://developers.google.com/ads-data-hub/guides/run-queries?hl=ja#parameter_types
#
PythonParameterType = (
    str
    | int
    | float
    | bool
    | datetime.date
    | datetime.datetime
    | list["PythonParameterType"]
)
"""
クエリのパラメータとして使える Python の型。
"""


def convert_param_types(
    params: dict[str, PythonParameterType],
) -> dict[str, ParameterTypeDict]:
    return {
        key: ParameterTypeDict({"type": convert_parameter_type(value)})
        for key, value in params.items()
    }


def convert_param_values(
    params: dict[str, PythonParameterType],
) -> dict[str, ParameterValueDict]:
    return {key: convert_parameter_value(value) for key, value in params.items()}


def convert_parameter_type(
    value: PythonParameterType,
) -> FieldTypeDict:
    match value:
        case str():
            return TypeDict({"type": "STRING"})

        case int():
            return TypeDict({"type": "INT64"})

        case float():
            return TypeDict({"type": "FLOAT64"})

        case bool():
            return TypeDict({"type": "BOOL"})

        case datetime.date():
            return TypeDict({"type": "DATE"})

        case datetime.datetime():
            return TypeDict({"type": "TIMESTAMP"})

        case list():
            return ArrayTypeDict(
                {
                    "arrayType": convert_parameter_type(
                        # NOTE: 配列の要素が空の場合は、型を推測できないため、文字列を仮に入れる。
                        #       クエリ文の側でパラメータの型が明確である場合、文字列は自動で変換処理されるため、最も安全な選択肢となる。
                        value[0] if len(value) > 0 else "STRING"
                    )
                }
            )

        case _ as unreachable:
            assert_never(unreachable)


def convert_parameter_value(value: PythonParameterType) -> ParameterValueDict:
    match value:
        case str():
            return ValueDict({"value": value})

        case int():
            return ValueDict({"value": str(value)})

        case float():
            return ValueDict({"value": str(value)})

        case bool():
            return ValueDict({"value": str(value)})

        case datetime.date():
            return ValueDict({"value": value.isoformat()})

        case datetime.datetime():
            return ValueDict({"value": value.isoformat()})

        case list():
            return ArrayValueDict(
                {
                    "arrayValue": ArrayValueValuesDict(
                        {"values": [convert_parameter_value(v) for v in value]}
                    )
                }
            )

        case _ as unreachable:
            assert_never(unreachable)



import datetime

from typing_extensions import override

import adsdatahub
from adsdatahub.client.customer import CustomerClient
from adsdatahub.client.parameters import PythonParameterType
from adsdatahub.client.query_result import QueryResult
from adsdatahub.types import CustomerId


class MockCustomerClient(CustomerClient):
    def __init__(
        self,
        client: adsdatahub.MockClient,
        customer_id: CustomerId,
    ):
        self._client = client
        self.customer_id = customer_id

    def query(
        self,
        query_text: str,
        /,
        parameters: dict[str, PythonParameterType] | None = None,
        *,
        start_date: str | datetime.date,
        end_date: str | datetime.date,
        dest_table: str,
    ) -> QueryResult:
        # TODO: モックのインターフェースを検討する。
        raise NotImplementedError()

    @override
    def validate(
        self,
        query_text: str,
        /,
        parameters: dict[str, PythonParameterType] | None = None,
    ) -> None:
        # TODO: モックのインターフェースを検討する。
        raise NotImplementedError()



import importlib.metadata

from .client import Client as Client
from .client import MockClient as MockClient
from .client import RealClient as RealClient

__version__ = importlib.metadata.version(__name__)



from typing import TYPE_CHECKING, Literal, TypedDict, Unpack

from adsdatahub.restapi._helpers import convert_json_value
from adsdatahub.restapi.resources.analysis_queries.list import (
    AnalysisQueryListQueryParams,
    AnalysisQueryListResponse,
)
from adsdatahub.restapi.resources.analysis_queries.start_transient import (
    AnalysisQueriesStartTransientQueryParams,
)
from adsdatahub.restapi.resources.analysis_queries.validate import (
    AnalysisQueriesValidateQueryParams,
    AnalysisQueriesValidateResponseBodyModel,
)
from adsdatahub.restapi.schemas.analysis_query import (
    AnalysisQueryModel,
    AnalysisQueryRequest,
    AnalysisQueryRequestModel,
    AnalysisQueryRequestOptionalTitleModel,
)
from adsdatahub.restapi.schemas.analysis_query_metadata import (
    AnalysisQueryMetadataModel,
)
from adsdatahub.restapi.schemas.operation import OperationModel
from adsdatahub.restapi.schemas.query_execution_spec import (
    QueryExecutionSpecRequestModel,
)
from adsdatahub.types import CustomerId

ResourceName = Literal[
    "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries"
]
RESOURCE_NAME: ResourceName = (
    "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries"
)

Method = Literal["CREATE", "LIST", "START_TRANSIENT", "VALIDATE"]

if TYPE_CHECKING:
    import adsdatahub.restapi.http


class PathParameters(TypedDict):
    customer_id: CustomerId


class Resource:
    def __init__(
        self, http: "adsdatahub.restapi.http.Client", path_parameters: PathParameters
    ) -> None:
        self._http = http
        self._base_url = RESOURCE_NAME.format(**path_parameters)

    def create(
        self,
        request_body: AnalysisQueryRequest,
    ) -> AnalysisQueryModel:
        """
        後で実行するための分析クエリを作成します。
        現時点では、クエリの検証は行われません。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/create?hl=ja
        """
        if isinstance(request_body, dict):
            request_body = AnalysisQueryRequestModel.model_validate(request_body)

        return self._http.request(
            "POST",
            self._base_url,
            AnalysisQueryModel,
            json=request_body.model_dump(),
        )

    def list(
        self,
        query_params: AnalysisQueryListQueryParams | None = None,
    ) -> AnalysisQueryListResponse:
        """
        指定した顧客が所有する分析クエリを一覧表示します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/list?hl=ja
        """
        return self._http.request(
            "GET",
            self._base_url,
            AnalysisQueryListResponse,
            params={k: v for k, v in (query_params or {}) if v is not None},
        )

    def start_transient(
        self, request_body: AnalysisQueriesStartTransientQueryParams
    ) -> OperationModel[AnalysisQueryMetadataModel]:
        """
        一時的な分析クエリで実行を開始します。
        結果は、指定した BigQuery 宛先テーブルに書き込まれます。
        返されたオペレーション名を使用して、クエリ完了ステータスをポーリングできます。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/startTransient?hl=ja
        """

        return self._http.request(
            "POST",
            f"{self._base_url}:startTransient",
            OperationModel[AnalysisQueryMetadataModel],
            json=convert_json_value(
                request_body,
                model_map={
                    "query": AnalysisQueryRequestOptionalTitleModel,
                    "spec": QueryExecutionSpecRequestModel,
                },
            ),
        )

    def validate(
        self, request_body: AnalysisQueriesValidateQueryParams
    ) -> AnalysisQueriesValidateResponseBodyModel:
        """
        提供された分析クエリに対して静的検証チェックを実行します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/validate?hl=ja
        """
        return self._http.request(
            "POST",
            f"{self._base_url}:validate",
            AnalysisQueriesValidateResponseBodyModel,
            json=convert_json_value(
                request_body,
                model_map={
                    "query": AnalysisQueryRequestOptionalTitleModel,
                    "spec": QueryExecutionSpecRequestModel,
                },
            ),
        )


class MockResource:
    def __init__(
        self,
        mock_client: "adsdatahub.restapi.MockClient",
        **path_parameters: Unpack[PathParameters],
    ) -> None:
        self._mock_client = mock_client
        self._base_url = RESOURCE_NAME.format(**path_parameters)

    def create(
        self, response: AnalysisQueryModel | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response("POST", self._base_url, response)

        return self._mock_client

    def list(
        self, response: AnalysisQueryListResponse | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response("GET", self._base_url, response)

        return self._mock_client

    def start_transient(
        self, response: OperationModel[AnalysisQueryMetadataModel] | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response(
            "POST", f"{self._base_url}:startTransient", response
        )

        return self._mock_client

    def validate(
        self, response: AnalysisQueriesValidateResponseBodyModel | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response(
            "POST", f"{self._base_url}:validate", response
        )

        return self._mock_client



import datetime
from typing import TYPE_CHECKING, Literal, TypedDict, Unpack

from adsdatahub.restapi.resources.operation.wait import OperationWaitRequestBody
from adsdatahub.restapi.schemas.analysis_query_metadata import (
    AnalysisQueryMetadataModel,
    AnalysisQueryMetadataWithQueryTextModel,
)
from adsdatahub.restapi.schemas.operation import OperationModel
from adsdatahub.types import OperationId

if TYPE_CHECKING:
    import adsdatahub.restapi.http

ResourceName = Literal["https://adsdatahub.googleapis.com/v1/operations/{operation_id}"]
RESOURCE_NAME: ResourceName = (
    "https://adsdatahub.googleapis.com/v1/operations/{operation_id}"
)


class PathParameters(TypedDict):
    operation_id: OperationId


class Resource:
    """
    このリソースは、ネットワーク API 呼び出しの結果である長時間実行オペレーションを表します。
    """

    def __init__(
        self, client: "adsdatahub.restapi.http.Client", path_parameters: PathParameters
    ) -> None:
        self._client = client
        self._base_url = RESOURCE_NAME.format(**path_parameters)

    def cancel(self) -> None:
        """
        長時間実行オペレーションの非同期キャンセルを開始します。
        サーバーは操作のキャンセルに全力を尽くしますが、成功は保証されません。
        このメソッドがサーバーでサポートされていない場合は、google.rpc.Code.UNIMPLEMENTED を返します。
        クライアントは Operations.GetOperation などのメソッドを使用して、キャンセルが成功したか、あるいはキャンセルしたにもかかわらずオペレーションが完了したかを確認できます。
        キャンセルが成功するとそのオペレーションは削除されず、google.rpc.Status.code が 1 の Operation.error 値を持つオペレーションになります。
        これは Code.CANCELLED に相当します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/cancel?hl=ja
        """
        return self._client.request("POST", f"{self._base_url}:cancel")

    def delete(self) -> None:
        """
        長時間実行オペレーションを削除します。
        このメソッドは、クライアントがそのオペレーション結果に関心がなくなったことを表しています。
        このメソッドではオペレーションはキャンセルされません。
        このメソッドがサーバーでサポートされていない場合は、google.rpc.Code.UNIMPLEMENTED を返します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/delete?hl=ja
        """

        return self._client.request("DELETE", self._base_url)

    def get(self) -> OperationModel[AnalysisQueryMetadataWithQueryTextModel]:
        """
        長時間実行オペレーションの最新の状態を取得します。
        クライアントはこのメソッドを使用して、API サービスで推奨される間隔でオペレーションの結果をポーリングできます。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/get?hl=ja
        """

        return self._client.request(
            "GET",
            self._base_url,
            OperationModel[AnalysisQueryMetadataWithQueryTextModel],
        )

    def wait(
        self, request_body: OperationWaitRequestBody | None = None
    ) -> OperationModel[AnalysisQueryMetadataModel]:
        """
        指定した長時間実行オペレーションが完了するか、指定したタイムアウトに達するまで待機し、最新の状態を返します。

        オペレーションがすでに完了している場合は、すぐに最新の状態が返されます。
        指定されたタイムアウトがデフォルトの HTTP/RPC タイムアウトを上回る場合は、HTTP/RPC タイムアウトが使用されます。
        サーバーがこのメソッドをサポートしていない場合は、google.rpc.Code.UNIMPLEMENTED を返します。
        このメソッドはベスト エフォートに基づきます。
        指定されたタイムアウト（直前を含む）の前に最新の状態を返すことがあります。
        つまり、すぐにレスポンスがあったとしても、オペレーションが完了したことを保証するものではありません。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/wait?hl=ja
        """
        request_body = request_body or {}

        if timeout := request_body.get("timeout"):
            match timeout:
                case str():
                    timeout_sec = timeout
                case int() | float():
                    timeout_sec = f"{timeout}s"
                case datetime.timedelta():
                    timeout_sec = f"{timeout.total_seconds()}s"
            request_body["timeout"] = timeout_sec

        return self._client.request(
            "POST",
            f"{self._base_url}:wait",
            OperationModel[AnalysisQueryMetadataModel],
            json=request_body,
        )


class MockResource:
    def __init__(
        self,
        mock_client: "adsdatahub.restapi.MockClient",
        **path_parameters: Unpack[PathParameters],
    ) -> None:
        self._mock_client = mock_client
        self._base_url = RESOURCE_NAME.format(**path_parameters)

    def cancel(
        self, response: None | Exception = None
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response(
            "POST", f"{self._base_url}:cancel", response
        )

        return self._mock_client

    def delete(
        self, response: None | Exception = None
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response("DELETE", self._base_url, response)

        return self._mock_client

    def get(
        self,
        response: OperationModel[AnalysisQueryMetadataWithQueryTextModel] | Exception,
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response("GET", self._base_url, response)

        return self._mock_client

    def wait(
        self, response: OperationModel[AnalysisQueryMetadataModel] | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response(
            "POST", f"{self._base_url}:wait", response
        )

        return self._mock_client



import json
from abc import abstractmethod
from typing import TYPE_CHECKING, Any

import httpx
from typing_extensions import override

from adsdatahub.types import OperationId

if TYPE_CHECKING:
    from adsdatahub.restapi.http.mock_client import StoreKey


class AdsDataHubException(Exception):
    """Base exception for adsdatahub."""

    @property
    @abstractmethod
    def message(self) -> str:
        """Return the exception message."""
        ...

    def __str__(self) -> str:
        return self.message


class AdsDataHubError(AdsDataHubException):
    """Base error for adsdatahub."""


class AdsDataHubResponseError(AdsDataHubError):
    pass


class AdsDataHubResponseStatusCodeError(AdsDataHubResponseError):
    """Response status code error for adsdatahub."""

    def __init__(self, response: httpx.Response) -> None:
        self.response = response
        if response.headers.get("content-type") == "application/json":
            self.response_body = response.json()
        else:
            self.response_body = response.content.decode("utf-8")

    @property
    @override
    def message(self) -> str:
        return f"Response Status Code Error {self.response}: {self.response_body}"


class AdsDataHubUnimplementedError(AdsDataHubResponseStatusCodeError):
    """
    このメソッドがサーバーでサポートされていないため、google.rpc.Code.UNIMPLEMENTED を返した場合のエラー。

    See: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/delete?hl=ja
    """

    def __init__(self, response: httpx.Response) -> None:
        super().__init__(response)


class AdsDataHubUnavailableError(AdsDataHubResponseStatusCodeError):
    """
    このメソッドがサーバーで利用できないため、google.rpc.Code.UNAVAILABLE を返した場合のエラー。

    See: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/delete?hl=ja
    """

    def __init__(self, response: httpx.Response) -> None:
        super().__init__(response)


class AdsDataHubResponseBodyHasError(AdsDataHubResponseError):
    def __init__(self, response: httpx.Response, error: dict[str, Any]) -> None:
        self.response = response
        self.error = error

    @property
    @override
    def message(self) -> str:
        return f"ResponseBody has error: {json.dumps(self.error, ensure_ascii=False)}"


class AdsDataHubMockError(AdsDataHubError):
    pass


class AdsDataHubMockDataTypeError(AdsDataHubMockError, TypeError):
    """Mock data type error for adsdatahub."""

    def __init__(self, response_type: type, expected_type: type) -> None:
        self.response_type = response_type
        self.expected_type = expected_type

    @property
    @override
    def message(self) -> str:
        return f"Mock Data Type Error {self.response_type.__name__}: expected {self.expected_type.__name__}"


class AdsDataHubMockStoreDataEmptyError(AdsDataHubMockError, ValueError):
    @property
    @override
    def message(self) -> str:
        return "Mock Store Data is empty"


class AdsDataHubMockStoreKeyError(AdsDataHubMockError, ValueError):
    def __init__(self, key: "StoreKey", expected_key: "StoreKey") -> None:
        self.key = key
        self.expected_key = expected_key

    @property
    @override
    def message(self) -> str:
        return f"Mock Store Key Error {self.key}: expected {self.expected_key}"


class AdsDataHubDestinationTableInfoNotFound(AdsDataHubError, ValueError):
    def __init__(self, operation_id: OperationId) -> None:
        self.operation_id = operation_id

    @property
    @override
    def message(self) -> str:
        return f"Destination Table Info Not Found: operation_id={self.operation_id}"



from functools import cache

from google.cloud import bigquery

from adsdatahub.exceptions import AdsDataHubDestinationTableInfoNotFound
from adsdatahub.restapi.schemas.analysis_query_metadata import (
    AnalysisQueryMetadataModel,
)
from adsdatahub.restapi.schemas.destination_table_info import DestinationTableInfoModel
from adsdatahub.restapi.schemas.operation import (
    OperationModel,
)


class QueryResult:
    def __init__(
        self,
        *,
        dest_table: str,
        operation: OperationModel[AnalysisQueryMetadataModel],
        job: bigquery.QueryJob,
    ) -> None:
        self.dest_table = dest_table
        self.operation = operation
        self.job = job

    @property
    @cache
    def table_info(self) -> DestinationTableInfoModel:
        if response := self.operation.response:
            for table in response.destination_tables:
                if table.table_path == self.dest_table:
                    return table

        raise AdsDataHubDestinationTableInfoNotFound(
            operation_id=self.operation.name.operation_id
        )



from pydantic import BaseModel, ConfigDict


class Model(BaseModel):
    pass


class ExtraAllowModel(Model):
    model_config = ConfigDict(extra="allow")


class ExtraForbidModel(Model):
    model_config = ConfigDict(extra="forbid")



import json
from typing import Any, Mapping, TypeVar

import httpx

from adsdatahub.exceptions import (
    AdsDataHubResponseBodyHasError,
    AdsDataHubResponseStatusCodeError,
    AdsDataHubUnavailableError,
    AdsDataHubUnimplementedError,
)
from adsdatahub.restapi.schemas._model import Model

GenericResponseBody = TypeVar("GenericResponseBody", bound=Model)


def convert_json_value(data: Mapping[str, Any], *, model_map: dict[str, type[Model]]):
    return {
        k: (
            (model_map[k].model_validate(v) if isinstance(v, dict) else v).model_dump()
            if k in model_map
            else v
        )
        for k, v in data.items()
    }


def validate_status_code(response: httpx.Response) -> None:
    if response.status_code != 200:
        match response.status_code:
            case 501:
                try:
                    if (
                        response.json().get("error", {}).get("status")
                        == "UNIMPLEMENTED"
                    ):
                        raise AdsDataHubUnimplementedError(response)
                except json.JSONDecodeError:
                    pass

            case 503:
                raise AdsDataHubUnavailableError(response)

        raise AdsDataHubResponseStatusCodeError(response)


def parse_response_body(
    response_body_type: type[GenericResponseBody],
    response: httpx.Response,
) -> GenericResponseBody:
    validate_status_code(response)

    data = response.json()

    if error := data.get("error"):
        raise AdsDataHubResponseBodyHasError(response, error)

    return response_body_type.model_validate(data)


def get_extra_fields(model: Model) -> dict[str, Any]:
    """
    Pydanticモデルにある未定義のフィールドを返却する。
    """

    extras = {}
    for key, value in model:
        if isinstance(value, Model):
            extra = get_extra_fields(value)
            if len(extra):
                extras[key] = extra

    for key, value in (model.__pydantic_extra__ or {}).items():
        if isinstance(value, Model):
            extras[key] = get_extra_fields(value)
        else:
            extras[key] = value

    return extras



from typing import Annotated

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.noise_impact import NoiseImpact


class ColumnInfoModel(ExtraAllowModel):
    """
    Metadata of a destination table column.

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/ColumnInfo
    """

    name: str
    """
    The column name
    """

    noise_impact: Annotated[NoiseImpact, Field(alias="noiseImpact")]
    """
    The noise impact message for this column.
    """

    impact_percentage: Annotated[float | None, Field(alias="impactPercentage")] = None
    """
    If applicable, the share this column contributes to the noise impact.
    """



from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.field_type import FieldTypeModel


class ColumnModel(ExtraAllowModel):
    columnId: str
    columnType: FieldTypeModel



import datetime

from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraForbidModel


class DateDict(TypedDict):
    """
    カレンダーの日付全体または一部（誕生日など）を表します。時間帯とタイムゾーンは他の場所で指定されているか、重要ではありません。日付はグレゴリオ暦に基づき、次のいずれかを表します。

    年、月、日の値が 0 以外の完全な日付。
    年と 0 年の年（記念日など）。
    自ら年（0 か月と 0 日）。
    年と月、ゼロ日（クレジット カードの有効期限など）

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/Date?hl=ja
    """

    year: int
    """日付の年。1 ～ 9999 の範囲で指定するか、年のない日付を指定する場合は 0 にする必要があります。"""

    month: int
    """月。1 ～ 12 の範囲で指定するか、特定の月を指定しない年（0）を指定します。"""

    day: int
    """日。その年と月で有効な値を 1 ～ 31 の範囲で指定するか、0: 1 年のみを指定するか、日付が重要ではない年と月を指定します。"""


class DateModel(ExtraForbidModel):
    year: int
    month: int
    day: int

    def to_date(self) -> datetime.date:
        return datetime.date(self.year, self.month, self.day)



from abc import abstractmethod
from typing import TYPE_CHECKING, Unpack

from google.cloud import bigquery

import adsdatahub.restapi
from adsdatahub.restapi.real_client import RealRestApiClientConstructerKwargs
from adsdatahub.types import CustomerId

if TYPE_CHECKING:
    from adsdatahub.client.customer import CustomerClient


class Client:
    def __new__(
        cls,
        restapi_client: adsdatahub.restapi.Client | None = None,
        bigquery_client: bigquery.Client | None = None,
        **kwargs: Unpack[RealRestApiClientConstructerKwargs],
    ) -> "Client":
        if cls is Client:
            from .real_client import RealClient

            return RealClient(restapi_client, bigquery_client, **kwargs)

        return super().__new__(cls)

    @abstractmethod
    def customer(self, customer_id: CustomerId, /) -> "CustomerClient":
        """
        クエリを実行するためのクライアントを生成する。
        """

        raise NotImplementedError



import datetime
from typing import Annotated

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.parameter_value import ParameterValueModel


class QueryMetadataBaseModel(ExtraAllowModel):
    """
    クエリ実行ジョブに関するメタデータ。
    これは、クエリ実行リクエストによって返される google.longrunning.Operation のメタデータフィールドに保存されます。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/QueryMetadata?hl=ja
    """

    type: Annotated[str, Field(alias="@type")]
    """
    タイプを識別する URI。
    """

    query_resource_name: Annotated[str | None, Field(alias="queryResourceName")] = None
    """
    実行されたクエリリソースの名前（例: customers/123/analysisQueries/abcd1234）
    格納されているクエリ実行に対してのみ存在します。
    """

    query_title: Annotated[str | None, Field(alias="queryTitle")] = None
    """
    実行されたクエリのタイトル。
    """

    customer_id: Annotated[str, Field(alias="customerId")]
    """
    クエリを実行した Ads Data Hub お客様 ID。
    """

    ads_data_customer_id: Annotated[str, Field(alias="adsDataCustomerId")]
    """
    広告データに使用される Ads Data Hub お客様 ID。
    """

    match_data_customer_id: Annotated[str, Field(alias="matchDataCustomerId")]
    """
    マッチテーブルのデータに使用する Ads Data Hub お客様 ID。
    """

    parameter_values: Annotated[
        dict[str, ParameterValueModel],
        Field(alias="parameterValues", default_factory=dict),
    ]
    start_time: Annotated[datetime.datetime, Field(alias="startTime")]
    """
    クエリ実行の開始時間。

    RFC3339 UTC の Zulu 形式のタイムスタンプ。
    ナノ秒単位で、小数点以下は 9 桁までとなります。
    （例: "2014-10-02T15:01:23Z"、"2014-10-02T15:01:23.045123456Z"）。
    """

    end_time: Annotated[datetime.datetime, Field(alias="endTime")]
    """
    クエリ実行の終了時間。

    RFC3339 UTC の Zulu 形式のタイムスタンプ。
    ナノ秒単位で、小数点以下は 9 桁までとなります。
    （例: "2014-10-02T15:01:23Z"、"2014-10-02T15:01:23.045123456Z"）。
    """

    dest_table: Annotated[str | None, Field(alias="destTable")] = None
    """
    クエリ結果の宛先テーブル。

    分析クエリに使用されます。
    """



from typing import Annotated, NotRequired, TypedDict

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.analysis_query_metadata import (
    AnalysisQueryMetadataModel,
)
from adsdatahub.restapi.schemas.operation import OperationModel


class OperationsListQueryParams(TypedDict):
    filter: NotRequired[str | None]
    pageSize: NotRequired[int | None]
    pageToken: NotRequired[str | None]


class OperationsListResponseBody(ExtraAllowModel):
    operations: Annotated[
        list[OperationModel[AnalysisQueryMetadataModel]], Field(default_factory=list)
    ]
    """
    リクエストで指定されたフィルタに一致するオペレーションのリスト。
    """

    next_page_token: Annotated[str | None, Field(alias="nextPageToken")] = None
    """
    標準的なリストの次ページのトークン。
    """



from typing import Any

from typing_extensions import override

from adsdatahub.client.client import Client
from adsdatahub.types import CustomerId


class MockClient(Client):
    def __init__(
        self,
        **kwargs: Any,
    ) -> None:
        pass

    @override
    def customer(self, customer_id: CustomerId):
        from adsdatahub.client.mock_customer import MockCustomerClient

        return MockCustomerClient(self, customer_id)



from enum import Enum


class SummaryType(str, Enum):
    """
    概要の種類。

    列の結合でサポートされているすべてのオペレーション。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#SummaryType
    """

    SUMMARY_TYPE_UNSPECIFIED = "SUMMARY_TYPE_UNSPECIFIED"
    """サマリータイプが指定されていません。概要列の値は NULL になります。"""

    CONSTANT = "CONSTANT"
    """マージされた列の値を、指定された定数に置き換えます。"""
    SUM = "SUM"
    """結合された列の値の合計を取得します。"""



from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.parameter_value import (
    ParameterValueDict,
    ParameterValueModel,
)
from adsdatahub.restapi.schemas.summary_type import SummaryType


class ColumnSummaryRuleDict(TypedDict):
    """
    単一の出力列の手順をマージします。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#ColumnSummaryRule
    """

    type: SummaryType
    value: ParameterValueDict


class ColumnSummaryRuleModel(ExtraAllowModel):
    type: SummaryType
    value: ParameterValueModel



from typing import TypeAlias

from httpx._types import TimeoutTypes as TimeoutTypes
from httpx._types import URLTypes as URLTypes

CustomerId: TypeAlias = str
"""
クエリを実行する顧客のID。基本的には数値の文字列だが、"global" という文字列が変えることもある。

ex. "1234567890", "global"
"""

AnalysisQueryId: TypeAlias = str
"""
クエリのリソースID。

ex. "00d30ae989fa487d9a96e15afc244770"
"""

OperationId: TypeAlias = str
"""
オペレーションの一意なID。

ex. "e3646136d4b14f92b5560286c20306f0"
"""



from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraForbidModel


class ValueDict(TypedDict):
    value: str


class ArrayValueDict(TypedDict):
    arrayValue: "ArrayValueValuesDict"


class ArrayValueValuesDict(TypedDict):
    values: list["ParameterValueDict"]


class StructValueDict(TypedDict):
    structValue: "StructValueValuesDict"


class StructValueValuesDict(TypedDict):
    values: dict[str, "ParameterValueDict"]


ParameterValueDict = ValueDict | ArrayValueDict | StructValueDict


class ValueModel(ExtraForbidModel):
    value: str


class ArrayValueModel(ExtraForbidModel):
    arrayValue: "ArrayValueValuesModel"


class ArrayValueValuesModel(ExtraForbidModel):
    values: list["ParameterValueModel"]


class StructValueModel(ExtraForbidModel):
    structValue: "StructValueValuesModel"


class StructValueValuesModel(ExtraForbidModel):
    values: dict[str, "ParameterValueModel"]


ParameterValueModel = ValueModel | ArrayValueModel | StructValueModel



from typing import Any, Union, Unpack, assert_never, overload

from typing_extensions import override

from adsdatahub.restapi import http
from adsdatahub.restapi.client import Client
from adsdatahub.restapi.resources import (
    analysis_queries,
    analysis_query,
    operation,
    operations,
)


class MockClient(Client):
    def __init__(self, **kwargs: Any) -> None:
        self._http_internal = http.MockClient()

    @property
    @override
    def _http(self) -> http.MockClient:
        return self._http_internal

    @overload
    def inject_response(
        self,
        resource_name: analysis_queries.ResourceName,
        **params: Unpack[analysis_queries.PathParameters],
    ) -> analysis_queries.MockResource: ...

    @overload
    def inject_response(
        self,
        resource_name: analysis_query.ResourceName,
        **params: Unpack[analysis_query.PathParameters],
    ) -> analysis_query.MockResource: ...

    @overload
    def inject_response(
        self,
        resource_name: operations.ResourceName,
        **params: Unpack[operations.PathParameters],
    ) -> operations.MockResource: ...

    @overload
    def inject_response(
        self,
        resource_name: operation.ResourceName,
        **params: Unpack[operation.PathParameters],
    ) -> operation.MockResource: ...

    def inject_response(
        self,
        resource_name: Union[
            analysis_queries.ResourceName,
            analysis_query.ResourceName,
            operations.ResourceName,
            operation.ResourceName,
        ],
        **params: Any,
    ) -> Union[
        analysis_queries.MockResource,
        analysis_query.MockResource,
        operations.MockResource,
        operation.MockResource,
    ]:
        match resource_name:
            case analysis_queries.RESOURCE_NAME:
                return analysis_queries.MockResource(self, **params)

            case analysis_query.RESOURCE_NAME:
                return analysis_query.MockResource(self, **params)

            case operations.RESOURCE_NAME:
                return operations.MockResource(self, **params)

            case operation.RESOURCE_NAME:
                return operation.MockResource(self, **params)

            case _ as unreachable:
                assert_never(unreachable)



from typing import Annotated

from pydantic import Field

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.temp_table import TempTableModel


class OutputArtifactsModel(ExtraAllowModel):
    """
    クエリの実行中に作成される出力アーティファクト。
    一時テーブルは、Ads Data Hub クエリで CREATE TABLE temp_table AS (...) を使用して作成されます。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/OutputArtifacts?hl=ja
    """

    temp_tables: Annotated[
        list[TempTableModel], Field(alias="tempTables", default_factory=list)
    ]



from enum import Enum


class PrivacyMessageType(str, Enum):
    """
    プライバシー メッセージ タイプを列挙する一意のコードです。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/PrivacyMessageType?hl=ja
    """

    PRIVACY_MESSAGE_TYPE_UNSPECIFIED = "PRIVACY_MESSAGE_TYPE_UNSPECIFIED"
    """プライバシー メッセージ コードが指定されていません。"""

    TOUCHPOINT_ANALYSIS_EXCLUSIONS = "TOUCHPOINT_ANALYSIS_EXCLUSIONS"
    """タッチポイントおよび/またはユーザーをタッチポイント分析から除外しました。"""

    MERGE_ROW_NO_DATA_FILTERED = "MERGE_ROW_NO_DATA_FILTERED"
    """このジョブでデータはフィルタリングされませんでした。"""

    MERGE_ROW_DATA_ON_FEW_USERS_FILTERED = "MERGE_ROW_DATA_ON_FEW_USERS_FILTERED"
    """データはフィルタされましたが、含まれるユーザーが少なすぎます。"""

    MERGE_ROW_ONE_ROW_WITH_MANY_USERS_FILTERED = (
        "MERGE_ROW_ONE_ROW_WITH_MANY_USERS_FILTERED"
    )
    """多数のユーザーに関する情報を含む 1 行がフィルタされました。"""

    MERGE_ROW_NO_MERGE_SPEC = "MERGE_ROW_NO_MERGE_SPEC"
    """データはフィルタリングされましたが、マージ行の指定がジョブに対して構成されていないため、マージ行が表示されませんでした。"""

    MERGE_ROW_ADDED = "MERGE_ROW_ADDED"
    """マージ行がジョブ出力に追加されました。"""

    MERGE_ROW_TEMPORARILY_UNAVAILABLE = "MERGE_ROW_TEMPORARILY_UNAVAILABLE"
    """行の統合に関するプライバシー メッセージは、プライバシーに関する考慮事項または抑制された統合行情報がないため、抑制されました。"""

    LARGE_CREDIT_OUTLIER_USERS_FILTERED = "LARGE_CREDIT_OUTLIER_USERS_FILTERED"
    """タッチポイント分析から外れたユーザーを除外した。"""

    PRIVACY_MESSAGE_TEMPORARILY_UNAVAILABLE = "PRIVACY_MESSAGE_TEMPORARILY_UNAVAILABLE"
    """プライバシー上の問題がある、または関連情報が利用できないためにメールが抑制された。"""

    DATA_ACCESS_BUDGET_IS_NEARLY_EXHAUSTED = "DATA_ACCESS_BUDGET_IS_NEARLY_EXHAUSTED"
    """データアクセス予算はほぼなくなりました。"""



from abc import abstractmethod
from typing import TYPE_CHECKING, Any, Union, assert_never, cast

from typing_extensions import Unpack, overload

from adsdatahub.restapi.resources import (
    analysis_queries,
    analysis_query,
    operation,
    operations,
)
from adsdatahub.types import TimeoutTypes

if TYPE_CHECKING:
    import adsdatahub.restapi.http

    from .real_client import RealRestApiClientConstructerKwargs


class Client:
    def __new__(
        cls, **kwargs: "Unpack[RealRestApiClientConstructerKwargs]"
    ) -> "Client":
        if cls is Client:
            from .real_client import RealClient

            return RealClient(**kwargs)

        return super().__new__(cls)

    @property
    @abstractmethod
    def _http(self) -> "adsdatahub.restapi.http.Client": ...

    @property
    def timeout(self) -> TimeoutTypes:
        return self._http.timeout

    @timeout.setter
    def timeout(self, value: TimeoutTypes) -> None:
        self._http.timeout = value

    @overload
    def resource(
        self,
        resource_name: analysis_queries.ResourceName,
        **params: Unpack[analysis_queries.PathParameters],
    ) -> analysis_queries.Resource:
        """
        Ads Data Hub 内で実行できる分析クエリを定義します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja
        """
        ...

    @overload
    def resource(
        self,
        resource_name: analysis_query.ResourceName,
        **params: Unpack[analysis_query.PathParameters],
    ) -> analysis_query.Resource:
        """
        Ads Data Hub 内で実行できる分析クエリを定義します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja
        """
        ...

    @overload
    def resource(
        self,
        resource_name: operations.ResourceName,
        **params: Unpack[operations.PathParameters],
    ) -> operations.Resource:
        """
        このリソースは、ネットワーク API 呼び出しの結果である長時間実行オペレーションを表します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations?hl=ja
        """
        ...

    @overload
    def resource(
        self,
        resource_name: operation.ResourceName,
        **params: Unpack[operation.PathParameters],
    ) -> operation.Resource:
        """
        このリソースは、ネットワーク API 呼び出しの結果である長時間実行オペレーションを表します。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations?hl=ja
        """
        ...

    def resource(
        self,
        resource_name: Union[
            analysis_queries.ResourceName,
            analysis_query.ResourceName,
            operations.ResourceName,
            operation.ResourceName,
        ],
        **params: Any,
    ) -> Union[
        analysis_queries.Resource,
        analysis_query.Resource,
        operations.Resource,
        operation.Resource,
    ]:
        match resource_name:
            case "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries":
                return analysis_queries.Resource(
                    self._http, cast(analysis_queries.PathParameters, params)
                )

            case "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries/{analysis_query_id}":
                return analysis_query.Resource(
                    self._http, cast(analysis_query.PathParameters, params)
                )

            case "https://adsdatahub.googleapis.com/v1/operations":
                return operations.Resource(
                    self._http, cast(operations.PathParameters, params)
                )

            case "https://adsdatahub.googleapis.com/v1/operations/{operation_id}":
                return operation.Resource(
                    self._http, cast(operation.PathParameters, params)
                )

            case _:
                assert_never(resource_name)



import datetime
from time import sleep

from typing_extensions import override

import adsdatahub
from adsdatahub.client.customer import CustomerClient
from adsdatahub.client.parameters import (
    PythonParameterType,
    convert_param_types,
    convert_param_values,
)
from adsdatahub.client.query_result import QueryResult
from adsdatahub.types import CustomerId


class RealCustomerClient(CustomerClient):
    def __init__(
        self,
        client: adsdatahub.RealClient,
        customer_id: CustomerId,
    ):
        self._client = client
        self.customer_id = customer_id

    @override
    def query(
        self,
        query_text: str,
        /,
        parameters: dict[str, PythonParameterType] | None = None,
        *,
        start_date: str | datetime.date,
        end_date: str | datetime.date,
        dest_table: str,
    ) -> QueryResult:
        operation = self._client.restapi.resource(
            "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries",
            customer_id=self.customer_id,
        ).start_transient(
            {
                "query": {
                    "queryText": query_text,
                    "parameterTypes": convert_param_types(parameters or {}),
                },
                "spec": {
                    "startDate": start_date,
                    "endDate": end_date,
                    "parameterValues": convert_param_values(parameters or {}),
                },
                "destTable": dest_table,
            }
        )

        while not (
            operation := self._client.restapi.resource(
                "https://adsdatahub.googleapis.com/v1/operations/{operation_id}",
                operation_id=operation.name.operation_id,
            ).wait()
        ).done:
            sleep(3)

        return QueryResult(
            dest_table=dest_table,
            operation=operation,
            job=self._client.bigquery_client.query(f"SELECT * FROM {dest_table}"),
        )

    @override
    def validate(
        self,
        query_text: str,
        /,
        parameters: dict[str, PythonParameterType] | None = None,
    ) -> None:
        self._client.restapi.resource(
            "https://adsdatahub.googleapis.com/v1/customers/{customer_id}/analysisQueries",
            customer_id=self.customer_id,
        ).validate(
            {
                "query": {
                    "queryText": query_text,
                    "parameterTypes": convert_param_types(parameters or {}),
                }
            }
        )



import datetime
from abc import ABC, abstractmethod

import adsdatahub
from adsdatahub.client.parameters import PythonParameterType
from adsdatahub.client.query_result import QueryResult
from adsdatahub.types import CustomerId


class CustomerClient(ABC):
    def __init__(
        self,
        client: adsdatahub.Client,
        customer_id: CustomerId,
    ): ...

    @abstractmethod
    def query(
        self,
        query_text: str,
        /,
        parameters: dict[str, PythonParameterType] | None = None,
        *,
        start_date: str | datetime.date,
        end_date: str | datetime.date,
        dest_table: str,
    ) -> QueryResult: ...

    @abstractmethod
    def validate(
        self,
        query_text: str,
        /,
        parameters: dict[str, PythonParameterType] | None = None,
    ) -> None: ...



import typing
from abc import ABCMeta, abstractmethod

from httpx._client import UseClientDefault
from httpx._types import (
    AuthTypes,
    CookieTypes,
    HeaderTypes,
    QueryParamTypes,
    RequestContent,
    RequestData,
    RequestExtensions,
    RequestFiles,
    TimeoutTypes,
    URLTypes,
)

from adsdatahub.restapi._helpers import GenericResponseBody


class HttpRequestKwargs(typing.TypedDict):
    content: typing.NotRequired[typing.Optional[RequestContent]]
    data: typing.NotRequired[typing.Optional[RequestData]]
    files: typing.NotRequired[typing.Optional[RequestFiles]]
    json: typing.NotRequired[typing.Optional[typing.Any]]
    params: typing.NotRequired[typing.Optional[QueryParamTypes]]
    headers: typing.NotRequired[typing.Optional[HeaderTypes]]
    cookies: typing.NotRequired[typing.Optional[CookieTypes]]
    auth: typing.NotRequired[typing.Union[AuthTypes, UseClientDefault, None]]
    follow_redirects: typing.NotRequired[typing.Union[bool, UseClientDefault]]
    timeout: typing.NotRequired[typing.Union[TimeoutTypes, UseClientDefault]]
    extensions: typing.NotRequired[typing.Optional[RequestExtensions]]


class Client(metaclass=ABCMeta):
    @property
    @abstractmethod
    def timeout(self) -> TimeoutTypes: ...

    @timeout.setter
    @abstractmethod
    def timeout(self, value: TimeoutTypes) -> None: ...

    @typing.overload
    @abstractmethod
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: type[GenericResponseBody],
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> GenericResponseBody: ...

    @typing.overload
    @abstractmethod
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: None = None,
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> None: ...

    @abstractmethod
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: typing.Optional[type[GenericResponseBody]] = None,
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> typing.Optional[GenericResponseBody]: ...



from enum import Enum


class MergeType(str, Enum):
    """
    列の結合でサポートされているすべてのオペレーション。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#MergeType
    """

    MERGE_TYPE_UNSPECIFIED = "MERGE_TYPE_UNSPECIFIED"
    """マージタイプが指定されていません。結合された列の値は NULL になります。"""

    CONSTANT = "CONSTANT"
    """マージされた列の値を、指定された定数に置き換えます。"""

    SUM = "SUM"
    """結合された列の値の合計を取得します。"""



from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.privacy_message_type import PrivacyMessageType


class PrivacyMessageModel(ExtraAllowModel):
    """
    プライバシー関連の情報、または警告メッセージ。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/PrivacyMessage?hl=ja
    """

    type: PrivacyMessageType
    """メッセージのタイプ。"""

    details: str
    """メッセージのテキスト。"""



from typing import Annotated

from pydantic import Field

from adsdatahub.restapi.schemas.query_metadata import QueryMetadataBaseModel


class AnalysisQueryMetadataModel(QueryMetadataBaseModel):
    """
    クエリ実行ジョブに関するメタデータ。
    これは、クエリ実行リクエストによって返される google.longrunning.Operation のメタデータフィールドに保存されます。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/QueryMetadata?hl=ja
    """

    dest_table: Annotated[str | None, Field(alias="destTable")] = None
    """
    クエリ結果の宛先テーブル。

    分析クエリに使用されます。
    """


class AnalysisQueryMetadataWithQueryTextModel(AnalysisQueryMetadataModel):
    query_text: Annotated[str | None, Field(alias="queryText")] = None
    """
    実行されたクエリのテキスト。

    このフィールドは、google.longrunning.Operations.GetOperation 呼び出しのレスポンスでのみ設定されます。
    google.longrunning.Operations.ListOperations の呼び出しや他の呼び出しのレスポンスでは省略されます。
    """



from typing import TYPE_CHECKING, Literal, TypedDict, Unpack

from adsdatahub.restapi.resources.operations.list import (
    OperationsListQueryParams,
    OperationsListResponseBody,
)

if TYPE_CHECKING:
    import adsdatahub.restapi.http


ResourceName = Literal["https://adsdatahub.googleapis.com/v1/operations"]
RESOURCE_NAME: ResourceName = "https://adsdatahub.googleapis.com/v1/operations"


class PathParameters(TypedDict):
    pass


class Resource:
    """
    このリソースは、ネットワーク API 呼び出しの結果である長時間実行オペレーションを表します。
    """

    def __init__(
        self, client: "adsdatahub.restapi.http.Client", path_params: PathParameters
    ) -> None:
        self._client = client
        self._base_url = RESOURCE_NAME.format(**path_params)

    def list(
        self, query_params: OperationsListQueryParams | None = None
    ) -> OperationsListResponseBody:
        """
        リクエストで指定されたフィルタに一致するオペレーションをリストします。
        このメソッドがサーバーでサポートされていない場合は、UNIMPLEMENTED を返します。

        注: name バインディングを使用すると、users/*/operations などの異なるリソース名スキームを使用するために、API サービスがバインディングをオーバーライドできます。
        バインディングをオーバーライドするときに、API サービスは "/v1/{name=users/*}/operations" のようなバインディングをサービス構成に追加する場合があります。
        下位互換性を維持するため、デフォルトの名前にはオペレーションのコレクション ID が含まれています。
        ただし、オーバーライドを行うユーザーは、名前のバインディングが親リソースであり、オペレーション コレクション ID がないことを確認する必要があります。

        Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations/list?hl=ja
        """

        return self._client.request(
            "GET",
            self._base_url,
            OperationsListResponseBody,
            params={k: v for k, v in (query_params or {}) if v is not None},
        )


class MockResource:
    def __init__(
        self,
        mock_client: "adsdatahub.restapi.MockClient",
        **path_parameters: Unpack[PathParameters],
    ) -> None:
        self._mock_client = mock_client
        self._base_url = RESOURCE_NAME.format(**path_parameters)

    def list(
        self, response_body: OperationsListResponseBody | Exception
    ) -> "adsdatahub.restapi.MockClient":
        self._mock_client._http.inject_response(
            "GET",
            self._base_url,
            response_body,
        )

        return self._mock_client



from typing import Annotated

from pydantic import Field
from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.merge_column import MergeColumnDict, MergeColumnModel


class MergeSpecDict(TypedDict):
    columns: dict[str, MergeColumnDict]


class MergeSpecModel(ExtraAllowModel):
    columns: Annotated[dict[str, MergeColumnModel], Field(default_factory=dict)]



from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.share_type import ShareType


class QueryShareDict(TypedDict):
    """
    所有する Ads Data Hub ユーザー以外のクエリも共有する方法を共有します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/QueryShare?hl=ja
    """

    shareType: ShareType


class QueryShareModel(ExtraAllowModel):
    shareType: ShareType



from typing_extensions import TypedDict

from adsdatahub.restapi.schemas.analysis_query import AnalysisQueryOptionalTitleRequest
from adsdatahub.restapi.schemas.query_execution_spec import QueryExecutionSpecRequest


class AnalysisQueriesStartTransientQueryParams(TypedDict):
    """
    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/startTransient?hl=ja#request-body
    """

    query: AnalysisQueryOptionalTitleRequest
    """
    実行するクエリ。
    """

    spec: QueryExecutionSpecRequest
    """
    クエリ実行パラメータを定義します。
    """

    destTable: str
    """
    'project.dataset.table_name' 形式のクエリ結果の宛先 BigQuery テーブル。

    指定するには、お客様の ADH アカウントのプロジェクトを明示的に許可リストに登録する必要があります。
    プロジェクトが指定されていない場合は、指定したお客様のデフォルトのプロジェクトが使用されます。
    プロジェクトもデータセットも指定しない場合、デフォルトのプロジェクトとデータセットが使用されます。
    """



from typing_extensions import NotRequired, TypedDict

from adsdatahub.restapi.schemas._model import ExtraForbidModel
from adsdatahub.restapi.schemas.query_execution_spec import (
    QueryExecutionSpecRequestDict,
    QueryExecutionSpecRequestModel,
)


class AnalysisQueriesStartDict(TypedDict):
    """
    保存された分析クエリの実行を開始します。
    結果は、指定した BigQuery 宛先テーブルに書き込まれます。
    返されたオペレーション名を使用して、クエリ完了ステータスをポーリングできます。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/start?hl=ja
    """

    spec: QueryExecutionSpecRequestDict
    """クエリ実行パラメータを定義します。"""

    destTable: str
    """
    'project.dataset.table_name' 形式のクエリ結果の宛先 BigQuery テーブル。
    指定するには、お客様の ADH アカウントのプロジェクトを明示的に許可リストに登録する必要があります。
    プロジェクトが指定されていない場合は、指定したお客様のデフォルトのプロジェクトが使用されます。
    プロジェクトもデータセットも指定しない場合、デフォルトのプロジェクトとデータセットが使用されます。
    """

    customerId: NotRequired[str | None]
    """
    Ads Data Hub のお客様がクエリを実行します。
    指定しない場合は、デフォルトでクエリを所有する顧客が対象となります。
    """


class AnalysisQueriesStartModel(ExtraForbidModel):
    spec: QueryExecutionSpecRequestModel
    destTable: str
    customerId: str | None



import typing

import httpx
from httpx._types import (
    HeaderTypes,
    URLTypes,
)
from typing_extensions import override

from adsdatahub.restapi._helpers import (
    GenericResponseBody,
    parse_response_body,
    validate_status_code,
)
from adsdatahub.restapi.http.client import Client, HttpRequestKwargs
from adsdatahub.types import TimeoutTypes


class RealClient(Client):
    def __init__(
        self, headers: typing.Optional[HeaderTypes] = None, timeout: TimeoutTypes = None
    ):
        self._client = httpx.Client(headers=headers, timeout=timeout)

    @property
    @override
    def timeout(self) -> TimeoutTypes:
        return self._client.timeout

    @timeout.setter
    @override
    def timeout(self, value: TimeoutTypes) -> None:
        self._client.timeout = value

    @typing.overload
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: type[GenericResponseBody],
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> GenericResponseBody: ...

    @typing.overload
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: None = None,
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> None: ...

    @override
    def request(
        self,
        method: str,
        url: URLTypes,
        response_body_type: typing.Optional[type[GenericResponseBody]] = None,
        **kwargs: typing.Unpack[HttpRequestKwargs],
    ) -> typing.Optional[GenericResponseBody]:
        response = self._client.request(method, url, **kwargs)

        if response_body_type is None:
            validate_status_code(response)

        else:
            return parse_response_body(
                response_body_type,
                response,
            )



from typing import Annotated

from pydantic import Field
from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.column_summary_rule import (
    ColumnSummaryRuleDict,
    ColumnSummaryRuleModel,
)


class FilteredRowSummaryDict(TypedDict):
    """
    分析クエリ結果のスキーマの手順を結合します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#FilteredRowSummary
    """

    columns: dict[str, ColumnSummaryRuleDict]


class FilteredRowSummaryModel(ExtraAllowModel):
    """
    分析クエリ結果のスキーマの手順を結合します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#FilteredRowSummary
    """

    columns: Annotated[dict[str, ColumnSummaryRuleModel], Field(default_factory=dict)]



from typing import NotRequired

from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraForbidModel

from .field_type import FieldType
from .parameter_value import ParameterValueDict, ParameterValueModel


class ParameterTypeDict(TypedDict):
    """
    特定のクエリ パラメータに関する情報。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/ParameterType?hl=ja
    """

    type: FieldType
    """
    パラメータ フィールドの型。
    """

    description: NotRequired[str | None]
    """
    パラメータの説明。
    """

    defaultValue: NotRequired[ParameterValueDict | None]
    """
    パラメータ値が指定されていない場合に使用する値。
    """


class ParameterTypeModel(ExtraForbidModel):
    type: FieldType
    """
    パラメータ フィールドの型。
    """

    description: str | None = None
    """
    パラメータの説明。
    """

    defaultValue: ParameterValueModel | None = None
    """
    パラメータ値が指定されていない場合に使用する値。
    """



from typing import Annotated

from pydantic import Field
from typing_extensions import TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel


class StatusDict(TypedDict):
    """
    Status 型は、REST API や RPC API など、さまざまなプログラミング環境に適した論理エラーモデルを定義します。
    gRPC により使用されます。
    各 Status メッセージには、エラーコード、エラー メッセージ、エラーの詳細という 3 種類のデータが含まれます。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations?hl=ja#Status
    """

    code: int
    """ステータス コード。google.rpc.Code の列挙値である必要があります。"""

    message: str
    """
    デベロッパー向けのエラー メッセージ。
    英語で記述します。
    ユーザー向けのエラー メッセージは、ローカライズして google.rpc.Status.details フィールドで送信するか、
    クライアントでローカライズする必要があります。
    """

    details: list[dict]
    """
    エラーの詳細を保持するメッセージのリスト。
    API が使用する共通のメッセージ タイプのセットがあります。

    任意のデータ型のフィールドを含むオブジェクト。
    タイプを識別する URI を含むフィールド "@type" を追加できます。

    例: { "id": 1234, "@type": "types.example.com/standard/id" }
    """


class StatusModel(ExtraAllowModel):
    code: int

    message: str

    details: Annotated[list[dict], Field(default_factory=list)]



import datetime

from pydantic import BeforeValidator, Field, PlainSerializer, ValidationInfo
from typing_extensions import Annotated, NotRequired, TypedDict

from adsdatahub.restapi.schemas._model import ExtraAllowModel, ExtraForbidModel
from adsdatahub.restapi.schemas.filtered_row_summary import (
    FilteredRowSummaryDict,
    FilteredRowSummaryModel,
)
from adsdatahub.restapi.schemas.merge_spec import MergeSpecDict, MergeSpecModel
from adsdatahub.restapi.schemas.parameter_type import (
    ParameterTypeDict,
    ParameterTypeModel,
)
from adsdatahub.restapi.schemas.query_share import QueryShareDict, QueryShareModel
from adsdatahub.restapi.schemas.query_state import QueryState
from adsdatahub.types import AnalysisQueryId, CustomerId


class AnalysisQueryNameModel(ExtraAllowModel):
    customer_id: CustomerId
    analysis_query_id: AnalysisQueryId

    def __str__(self) -> str:
        return f"customers/{self.customer_id}/analysisQueries/{self.analysis_query_id}"


def _deserialize_name(
    value: AnalysisQueryNameModel | str | None, info: ValidationInfo
) -> dict[str, str | int] | None:
    if value is None:
        return None
    if isinstance(value, AnalysisQueryNameModel):
        return {
            "customer_id": value.customer_id,
            "analysis_query_id": value.analysis_query_id,
        }

    splited_value = value.split("/")

    if (
        len(splited_value) != 4
        or splited_value[0] != "customers"
        or splited_value[2] != "analysisQueries"
    ):
        raise ValueError(f"Invalid operation name: {value}")

    return {
        "customer_id": splited_value[1],
        "analysis_query_id": splited_value[3],
    }


def _serialize_name(model: AnalysisQueryNameModel | None) -> str | None:
    if model is None:
        return None

    return f"customers/{model.customer_id}/analysisQueries/{model.analysis_query_id}"


class AnalysisQueryRequestDict(TypedDict):
    """
    Ads Data Hub 内で実行できる分析クエリを定義します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#AnalysisQuery
    """

    name: NotRequired[str | None]
    """クエリを一意に識別する名前。"""

    title: str
    """クエリのタイトル。Ads Data Hub の単一顧客とクエリタイプ内で一意です。"""

    queryText: str
    """標準 SQL で記述されたクエリテキスト。"""

    parameterTypes: NotRequired[dict[str, ParameterTypeDict] | None]
    """クエリで想定されるその他のパラメータ。各引数名をその引数タイプにマッピングします。"""

    mergeSpec: NotRequired[MergeSpecDict | None]
    """
    行をマージする手順。
    存在する場合、プライバシー上の理由でドロップされるはずの行が 1 つに結合されます。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に表示されます。
    """

    queryState: NotRequired[QueryState | str | None]
    """クエリの状態。"""

    queryShare: NotRequired[list[QueryShareDict] | None]
    """所有する Ads Data Hub ユーザー以外のクエリも共有する方法を紹介します。"""

    filteredRowSummary: NotRequired[FilteredRowSummaryDict | None]
    """
    プライバシー上の理由によってドロップされた行を 1 つの結合行に統合する方法を定義します。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に含められます。

    generateFilteredRowSummaryAutomatically と同時に使えません。"""

    generateFilteredRowSummaryAutomatically: NotRequired[bool | None]
    """
    true の場合、フィルタリングされた行の概要が自動的に生成されます。

    filteredRowSummary と同時に使えません。
    """


class AnalysisQueryRequestOptionalTitleDict(TypedDict):
    """
    Ads Data Hub 内で実行できる分析クエリを定義します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#AnalysisQuery
    """

    name: NotRequired[str | None]
    """
    クエリを一意に識別する名前。
    """

    title: NotRequired[str | None]
    """
    クエリのタイトル。

    Ads Data Hub の単一顧客とクエリタイプ内で一意です。
    """

    queryText: str
    """
    標準 SQL で記述されたクエリテキスト。
    """

    parameterTypes: NotRequired[dict[str, ParameterTypeDict] | None]
    """
    クエリで想定されるその他のパラメータ。

    各引数名をその引数タイプにマッピングします。
    """

    mergeSpec: NotRequired[MergeSpecDict | None]
    """
    行をマージする手順。

    存在する場合、プライバシー上の理由でドロップされるはずの行が 1 つに結合されます。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に表示されます。
    """

    queryState: NotRequired[QueryState | str | None]
    """
    クエリの状態。
    """

    queryShare: NotRequired[list[QueryShareDict] | None]
    """
    所有する Ads Data Hub ユーザー以外のクエリも共有する方法を紹介します。
    """

    filteredRowSummary: NotRequired[FilteredRowSummaryDict | None]
    """
    プライバシー上の理由によってドロップされた行を 1 つの結合行に統合する方法を定義します。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に含められます。

    generateFilteredRowSummaryAutomatically と同時に使えません。
    """

    generateFilteredRowSummaryAutomatically: NotRequired[bool | None]
    """
    true の場合、フィルタリングされた行の概要が自動的に生成されます。

    filteredRowSummary と同時に使えません。
    """


class AnalysisQueryRequestBaseModel(ExtraForbidModel):
    """
    Ads Data Hub 内で実行できる分析クエリを定義します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#AnalysisQuery
    """

    name: Annotated[
        AnalysisQueryNameModel | None,
        BeforeValidator(_deserialize_name),
        PlainSerializer(_serialize_name),
    ] = None
    """
    クエリを一意に識別する名前。
    """

    query_text: Annotated[str, Field(alias="queryText")]
    """
    標準 SQL で記述されたクエリテキスト。
    """
    parameter_types: Annotated[
        dict[str, ParameterTypeModel],
        Field(
            alias="parameterTypes",
            default_factory=dict,
        ),
    ]
    """
    クエリで想定されるその他のパラメータ。

    各引数名をその引数タイプにマッピングします。
    """

    merge_spec: Annotated[
        MergeSpecModel,
        Field(alias="mergeSpec", default_factory=lambda: MergeSpecModel(columns={})),
    ]
    """
    行をマージする手順。

    deprecated: このフィールドは非推奨です。代わりに filter_row_summary を使用してください。

    存在する場合、プライバシー上の理由でドロップされるはずの行が 1 つに結合されます。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に表示されます。
    """

    queryState: Annotated[QueryState | str | None, Field(alias="queryState")] = None
    """
    クエリの状態。
    """

    query_share: Annotated[
        list[QueryShareModel], Field(alias="queryShare", default_factory=list)
    ]
    """
    所有する Ads Data Hub ユーザー以外のクエリも共有する方法を紹介します。
    """

    filtered_row_summary: Annotated[
        FilteredRowSummaryModel | None, Field(alias="filteredRowSummary")
    ] = None
    """
    プライバシー上の理由によってドロップされた行を 1 つの結合行に統合する方法を定義します。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に含められます。

    generateFilteredRowSummaryAutomatically と同時に使えません。
    """

    generate_filtered_row_summary_automatically: Annotated[
        bool | None, Field(alias="generateFilteredRowSummaryAutomatically")
    ] = None
    """
    true の場合、フィルタリングされた行の概要が自動的に生成されます。

    filteredRowSummary と同時に使えません。
    """


class AnalysisQueryRequestModel(AnalysisQueryRequestBaseModel):
    title: str
    """
    クエリのタイトル。

    Ads Data Hub の単一顧客とクエリタイプ内で一意です。
    """


class AnalysisQueryRequestOptionalTitleModel(AnalysisQueryRequestBaseModel):
    title: str | None = None
    """
    クエリのタイトル。

    Ads Data Hub の単一顧客とクエリタイプ内で一意です。
    """


class AnalysisQueryModel(ExtraAllowModel):
    """
    Ads Data Hub 内で実行できる分析クエリを定義します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries?hl=ja#AnalysisQuery
    """

    name: Annotated[
        AnalysisQueryNameModel,
        BeforeValidator(_deserialize_name),
        PlainSerializer(_serialize_name),
    ]

    title: str
    """
    クエリのタイトル。

    Ads Data Hub の単一顧客とクエリタイプ内で一意です。
    """

    query_text: Annotated[str | None, Field(alias="queryText")] = None
    """
    標準 SQL で記述されたクエリテキスト。
    """

    parameter_types: Annotated[
        dict[str, ParameterTypeModel],
        Field(
            alias="parameterTypes",
            default_factory=dict,
        ),
    ]
    """
    クエリで想定されるその他のパラメータ。

    各引数名をその引数タイプにマッピングします。
    """

    merge_spec: Annotated[
        MergeSpecModel,
        Field(alias="mergeSpec", default_factory=lambda: MergeSpecDict(columns={})),
    ]
    """
    行をマージする手順。

    deprecated: このフィールドは非推奨です。代わりに filter_row_summary を使用してください。

    存在する場合、プライバシー上の理由でドロップされるはずの行が 1 つに結合されます。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に表示されます。
    """

    query_state: Annotated[QueryState, Field(alias="queryState")]
    """
    クエリの状態。
    """

    update_time: Annotated[datetime.datetime, Field(alias="updateTime")]

    update_email: Annotated[str, Field(alias="updateEmail")]

    create_time: Annotated[datetime.datetime, Field(alias="createTime")]

    create_email: Annotated[str | None, Field(alias="createEmail")] = None

    query_share: Annotated[list[QueryShareDict], Field(default_factory=list)]
    """
    所有する Ads Data Hub ユーザー以外のクエリも共有する方法を紹介します。
    """

    filtered_row_summary: Annotated[
        FilteredRowSummaryModel | None, Field(alias="filteredRowSummary")
    ] = None
    """
    プライバシー上の理由によってドロップされた行を 1 つの結合行に統合する方法を定義します。
    マージされた行がプライバシー要件を満たしている場合は、マージされた行が最終出力に含められます。

    generateFilteredRowSummaryAutomatically と同時に使えません。
    """

    generate_filtered_row_summary_automatically: Annotated[
        bool | None, Field(alias="generateFilteredRowSummaryAutomatically")
    ] = None
    """
    true の場合、フィルタリングされた行の概要が自動的に生成されます。

    filteredRowSummary と同時に使えません。
    """


AnalysisQueryRequest = AnalysisQueryRequestModel | AnalysisQueryRequestDict
AnalysisQueryOptionalTitleRequest = (
    AnalysisQueryRequestOptionalTitleModel | AnalysisQueryRequestOptionalTitleDict
)



from typing import Annotated, Generic, TypeVar

from pydantic import BeforeValidator, PlainSerializer, ValidationInfo

from adsdatahub.restapi.schemas._model import ExtraAllowModel
from adsdatahub.restapi.schemas.analysis_query_metadata import (
    AnalysisQueryMetadataModel,
)
from adsdatahub.restapi.schemas.query_response import QueryResponseModel
from adsdatahub.restapi.schemas.status import StatusModel
from adsdatahub.types import OperationId

GenericAnalysisQueryMetadataModel = TypeVar(
    "GenericAnalysisQueryMetadataModel", bound=AnalysisQueryMetadataModel
)


class OperationNameModel(ExtraAllowModel):
    operation_id: OperationId

    def __str__(self) -> str:
        return f"operations/{self.operation_id}"


def _deserialize_name(value: str, info: ValidationInfo) -> dict[str, str]:
    if not value.startswith("operations/"):
        raise ValueError(f"Invalid operation name: {value}")

    return {"operation_id": value[len("operations/") :]}


def _serialize_name(model: OperationNameModel) -> str:
    return f"operations/{model.operation_id}"


class OperationModel(ExtraAllowModel, Generic[GenericAnalysisQueryMetadataModel]):
    """
    このリソースは、ネットワーク API 呼び出しの結果である長時間実行オペレーションを表します。

    Reference: https://developers.google.com/ads-data-hub/reference/rest/v1/operations?hl=ja#Operation
    """

    name: Annotated[
        OperationNameModel,
        BeforeValidator(_deserialize_name),
        PlainSerializer(_serialize_name),
    ]
    """
    サーバーによって割り当てられる名前。

    最初にその名前を返すサービスと同じサービス内でのみ一意になります。
    デフォルトの HTTP マッピングを使用している場合は、name を operations/{operation_id} で終わるリソース名にします。
    """

    metadata: GenericAnalysisQueryMetadataModel | None
    """
    オペレーションに関連付けられたサービス固有のデータを含む QueryMetadata オブジェクト。

    任意のデータ型のフィールドを含むオブジェクト。
    タイプを識別する URI を含むフィールド "@type" を追加できます。

    例: { "id": 1234, "@type": "types.example.com/standard/id" }
    """

    done: bool = False
    """
    値が false の場合は、オペレーションが進行中であることを意味します。

    true の場合、オペレーションは完了しており、error または response が利用可能です。
    """

    error: StatusModel | None = None
    """
    失敗またはキャンセルされた場合のオペレーションのエラー結果。
    """

    response: QueryResponseModel | None = None
    """
    クエリ オペレーションが成功した場合に返される QueryResponse オブジェクト。

    任意のデータ型のフィールドを含むオブジェクト。
    タイプを識別する URI を含むフィールド "@type" を追加できます。

    例: { "id": 1234, "@type": "types.example.com/standard/id" }
    """



from enum import Enum


class QueryState(str, Enum):
    """
    分析クエリの実行モードを決定する状態。

    See https://developers.google.com/ads-data-hub/reference/rest/v1/QueryState?hl=ja
    """

    QUERY_STATE_UNSPECIFIED = "QUERY_STATE_UNSPECIFIED"
    """クエリの状態は指定されていません。"""

    RUNNABLE = "RUNNABLE"
    """クエリはお客様が実行できます。"""

    WHITELISTED = "WHITELISTED"
    """クエリは手動で安全であることが確認されています。"""
